/*!
* suni v1.1.0
* (c) 2019-present Yahtnif <yahtnif@gmail.com>
* Released under the Anti 996 License.
*/
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function chunks (array, perChunk) {
    return array.reduce(function (all, one, i) {
        var ch = Math.floor(i / perChunk);
        all[ch] = [].concat(all[ch] || [], one);
        return all;
    }, []);
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var defaultLabels = {
    count: 'count',
    value: 'value'
};
function count (arr, labels) {
    var counts = {};
    var _a = labels
        ? __assign(__assign({}, defaultLabels), labels) : defaultLabels, count = _a.count, value = _a.value;
    arr.forEach(function (value) {
        if (typeof value !== 'string')
            return;
        counts[value] ? counts[value]++ : (counts[value] = 1);
    });
    return Object.keys(counts)
        .map(function (key) {
        var _a;
        return (_a = {},
            _a["" + value] = key,
            _a["" + count] = counts[key],
            _a);
    })
        .sort(function (a, b) { return b[count] - a[count]; });
}

function create (n, start) {
    return Array.from(new Array(n), function (v, i) {
        return typeof start !== 'undefined'
            ? typeof start === 'number'
                ? i + start
                : start(i + 1)
            : i + 1;
    });
}

function shuffle (array) {
    var counter = array.length;
    while (counter > 0) {
        var index = Math.floor(Math.random() * counter);
        counter--;
        var temp = array[counter];
        array[counter] = array[index];
        array[index] = temp;
    }
    return array;
}

function unique (arr) {
    return Array.from(new Set(arr));
}

var Arr = {
    chunks: chunks,
    count: count,
    create: create,
    shuffle: shuffle,
    unique: unique
};

function safeAdd(x, y) {
    var lsw = (x & 0xffff) + (y & 0xffff);
    var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
    return (msw << 16) | (lsw & 0xffff);
}
function bitRotateLeft(num, cnt) {
    return (num << cnt) | (num >>> (32 - cnt));
}
function md5cmn(q, a, b, x, s, t) {
    return safeAdd(bitRotateLeft(safeAdd(safeAdd(a, q), safeAdd(x, t)), s), b);
}
function md5ff(a, b, c, d, x, s, t) {
    return md5cmn((b & c) | (~b & d), a, b, x, s, t);
}
function md5gg(a, b, c, d, x, s, t) {
    return md5cmn((b & d) | (c & ~d), a, b, x, s, t);
}
function md5hh(a, b, c, d, x, s, t) {
    return md5cmn(b ^ c ^ d, a, b, x, s, t);
}
function md5ii(a, b, c, d, x, s, t) {
    return md5cmn(c ^ (b | ~d), a, b, x, s, t);
}
function binlMD5(x, len) {
    x[len >> 5] |= 0x80 << len % 32;
    x[(((len + 64) >>> 9) << 4) + 14] = len;
    var i;
    var olda;
    var oldb;
    var oldc;
    var oldd;
    var a = 1732584193;
    var b = -271733879;
    var c = -1732584194;
    var d = 271733878;
    for (i = 0; i < x.length; i += 16) {
        olda = a;
        oldb = b;
        oldc = c;
        oldd = d;
        a = md5ff(a, b, c, d, x[i], 7, -680876936);
        d = md5ff(d, a, b, c, x[i + 1], 12, -389564586);
        c = md5ff(c, d, a, b, x[i + 2], 17, 606105819);
        b = md5ff(b, c, d, a, x[i + 3], 22, -1044525330);
        a = md5ff(a, b, c, d, x[i + 4], 7, -176418897);
        d = md5ff(d, a, b, c, x[i + 5], 12, 1200080426);
        c = md5ff(c, d, a, b, x[i + 6], 17, -1473231341);
        b = md5ff(b, c, d, a, x[i + 7], 22, -45705983);
        a = md5ff(a, b, c, d, x[i + 8], 7, 1770035416);
        d = md5ff(d, a, b, c, x[i + 9], 12, -1958414417);
        c = md5ff(c, d, a, b, x[i + 10], 17, -42063);
        b = md5ff(b, c, d, a, x[i + 11], 22, -1990404162);
        a = md5ff(a, b, c, d, x[i + 12], 7, 1804603682);
        d = md5ff(d, a, b, c, x[i + 13], 12, -40341101);
        c = md5ff(c, d, a, b, x[i + 14], 17, -1502002290);
        b = md5ff(b, c, d, a, x[i + 15], 22, 1236535329);
        a = md5gg(a, b, c, d, x[i + 1], 5, -165796510);
        d = md5gg(d, a, b, c, x[i + 6], 9, -1069501632);
        c = md5gg(c, d, a, b, x[i + 11], 14, 643717713);
        b = md5gg(b, c, d, a, x[i], 20, -373897302);
        a = md5gg(a, b, c, d, x[i + 5], 5, -701558691);
        d = md5gg(d, a, b, c, x[i + 10], 9, 38016083);
        c = md5gg(c, d, a, b, x[i + 15], 14, -660478335);
        b = md5gg(b, c, d, a, x[i + 4], 20, -405537848);
        a = md5gg(a, b, c, d, x[i + 9], 5, 568446438);
        d = md5gg(d, a, b, c, x[i + 14], 9, -1019803690);
        c = md5gg(c, d, a, b, x[i + 3], 14, -187363961);
        b = md5gg(b, c, d, a, x[i + 8], 20, 1163531501);
        a = md5gg(a, b, c, d, x[i + 13], 5, -1444681467);
        d = md5gg(d, a, b, c, x[i + 2], 9, -51403784);
        c = md5gg(c, d, a, b, x[i + 7], 14, 1735328473);
        b = md5gg(b, c, d, a, x[i + 12], 20, -1926607734);
        a = md5hh(a, b, c, d, x[i + 5], 4, -378558);
        d = md5hh(d, a, b, c, x[i + 8], 11, -2022574463);
        c = md5hh(c, d, a, b, x[i + 11], 16, 1839030562);
        b = md5hh(b, c, d, a, x[i + 14], 23, -35309556);
        a = md5hh(a, b, c, d, x[i + 1], 4, -1530992060);
        d = md5hh(d, a, b, c, x[i + 4], 11, 1272893353);
        c = md5hh(c, d, a, b, x[i + 7], 16, -155497632);
        b = md5hh(b, c, d, a, x[i + 10], 23, -1094730640);
        a = md5hh(a, b, c, d, x[i + 13], 4, 681279174);
        d = md5hh(d, a, b, c, x[i], 11, -358537222);
        c = md5hh(c, d, a, b, x[i + 3], 16, -722521979);
        b = md5hh(b, c, d, a, x[i + 6], 23, 76029189);
        a = md5hh(a, b, c, d, x[i + 9], 4, -640364487);
        d = md5hh(d, a, b, c, x[i + 12], 11, -421815835);
        c = md5hh(c, d, a, b, x[i + 15], 16, 530742520);
        b = md5hh(b, c, d, a, x[i + 2], 23, -995338651);
        a = md5ii(a, b, c, d, x[i], 6, -198630844);
        d = md5ii(d, a, b, c, x[i + 7], 10, 1126891415);
        c = md5ii(c, d, a, b, x[i + 14], 15, -1416354905);
        b = md5ii(b, c, d, a, x[i + 5], 21, -57434055);
        a = md5ii(a, b, c, d, x[i + 12], 6, 1700485571);
        d = md5ii(d, a, b, c, x[i + 3], 10, -1894986606);
        c = md5ii(c, d, a, b, x[i + 10], 15, -1051523);
        b = md5ii(b, c, d, a, x[i + 1], 21, -2054922799);
        a = md5ii(a, b, c, d, x[i + 8], 6, 1873313359);
        d = md5ii(d, a, b, c, x[i + 15], 10, -30611744);
        c = md5ii(c, d, a, b, x[i + 6], 15, -1560198380);
        b = md5ii(b, c, d, a, x[i + 13], 21, 1309151649);
        a = md5ii(a, b, c, d, x[i + 4], 6, -145523070);
        d = md5ii(d, a, b, c, x[i + 11], 10, -1120210379);
        c = md5ii(c, d, a, b, x[i + 2], 15, 718787259);
        b = md5ii(b, c, d, a, x[i + 9], 21, -343485551);
        a = safeAdd(a, olda);
        b = safeAdd(b, oldb);
        c = safeAdd(c, oldc);
        d = safeAdd(d, oldd);
    }
    return [a, b, c, d];
}
function binl2rstr(input) {
    var i;
    var output = '';
    var length32 = input.length * 32;
    for (i = 0; i < length32; i += 8) {
        output += String.fromCharCode((input[i >> 5] >>> i % 32) & 0xff);
    }
    return output;
}
function rstr2binl(input) {
    var i;
    var output = [];
    output[(input.length >> 2) - 1] = undefined;
    for (i = 0; i < output.length; i += 1) {
        output[i] = 0;
    }
    var length8 = input.length * 8;
    for (i = 0; i < length8; i += 8) {
        output[i >> 5] |= (input.charCodeAt(i / 8) & 0xff) << i % 32;
    }
    return output;
}
function rstrMD5(s) {
    return binl2rstr(binlMD5(rstr2binl(s), s.length * 8));
}
function rstrHMACMD5(key, data) {
    var i;
    var bkey = rstr2binl(key);
    var ipad = [];
    var opad = [];
    var hash;
    ipad[15] = opad[15] = undefined;
    if (bkey.length > 16) {
        bkey = binlMD5(bkey, key.length * 8);
    }
    for (i = 0; i < 16; i += 1) {
        ipad[i] = bkey[i] ^ 0x36363636;
        opad[i] = bkey[i] ^ 0x5c5c5c5c;
    }
    hash = binlMD5(ipad.concat(rstr2binl(data)), 512 + data.length * 8);
    return binl2rstr(binlMD5(opad.concat(hash), 512 + 128));
}
function rstr2hex(input) {
    var hexTab = '0123456789abcdef';
    var output = '';
    var x;
    var i;
    for (i = 0; i < input.length; i += 1) {
        x = input.charCodeAt(i);
        output += hexTab.charAt((x >>> 4) & 0x0f) + hexTab.charAt(x & 0x0f);
    }
    return output;
}
function str2rstrUTF8(input) {
    return unescape(encodeURIComponent(input));
}
function rawMD5(s) {
    return rstrMD5(str2rstrUTF8(s));
}
function hexMD5(s) {
    return rstr2hex(rawMD5(s));
}
function rawHMACMD5(k, d) {
    return rstrHMACMD5(str2rstrUTF8(k), str2rstrUTF8(d));
}
function hexHMACMD5(k, d) {
    return rstr2hex(rawHMACMD5(k, d));
}
function md5(str, key, raw) {
    if (!key) {
        if (!raw) {
            return hexMD5(str);
        }
        return rawMD5(str);
    }
    if (!raw) {
        return hexHMACMD5(key, str);
    }
    return rawHMACMD5(key, str);
}

function pad(hash, len) {
    while (hash.length < len) {
        hash = '0' + hash;
    }
    return hash;
}
function fold(hash, text) {
    if (text.length === 0) {
        return hash;
    }
    for (var i = 0; i < text.length; i++) {
        var chr = text.charCodeAt(i);
        hash = (hash << 5) - hash + chr;
        hash |= 0;
    }
    return hash < 0 ? hash * -2 : hash;
}
function foldObject(hash, o, seen) {
    return Object.keys(o)
        .sort()
        .reduce(foldKey, hash);
    function foldKey(hash, key) {
        return foldValue(hash, o[key], key, seen);
    }
}
function foldValue(input, value, key, seen) {
    var hash = fold(fold(fold(input, key), toString(value)), typeof value);
    if (value === null) {
        return fold(hash, 'null');
    }
    if (value === undefined) {
        return fold(hash, 'undefined');
    }
    if (typeof value === 'object') {
        if (seen.indexOf(value) !== -1) {
            return fold(hash, '[Circular]' + key);
        }
        seen.push(value);
        return foldObject(hash, value, seen);
    }
    return fold(hash, value.toString());
}
function toString(o) {
    return Object.prototype.toString.call(o);
}
function sum (o) {
    return pad(foldValue(0, o, '', []).toString(16), 8);
}

var Hash = { md5: md5, sum: sum };

var has = Object.prototype.hasOwnProperty;
var getProperties = Object.getOwnPropertyNames;
var toString$1 = Object.prototype.toString;
function empty (val) {
    if (val == null)
        return true;
    if ('boolean' === typeof val)
        return false;
    if ('number' === typeof val)
        return false;
    if ('string' === typeof val)
        return val.length === 0;
    if ('function' === typeof val)
        return false;
    if (Array.isArray(val))
        return val.length === 0;
    if (val instanceof Error)
        return val.message === '';
    if (val.toString == toString$1) {
        switch (val.toString()) {
            case '[object File]':
            case '[object Map]':
            case '[object Set]': {
                return val.size === 0;
            }
            case '[object Object]': {
                if (getProperties)
                    return !getProperties(val).length;
                for (var key in val) {
                    if (has.call(val, key))
                        return false;
                }
                return true;
            }
        }
    }
    return false;
}

function promise (obj) {
    return (!!obj &&
        (typeof obj === 'object' || typeof obj === 'function') &&
        typeof obj.then === 'function');
}

var index = { empty: empty, promise: promise };

function filter(obj, predicate) {
    var ret = {};
    var keys = Object.keys(obj);
    var isArray = Array.isArray(predicate);
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        var val = obj[key];
        if (isArray
            ? predicate.indexOf(key) !== -1
            : predicate(key, val, obj)) {
            ret[key] = val;
        }
    }
    return ret;
}

function get (obj, key, def) {
    var p = 0;
    key = key.split ? key.split('.') : key;
    while (obj && p < key.length)
        obj = obj[key[p++]];
    return obj === undefined || p < key.length ? def : obj;
}

var isObject = function (x) {
    return typeof x === 'object' &&
        x !== null &&
        !(x instanceof RegExp) &&
        !(x instanceof Error) &&
        !(x instanceof Date);
};
function mapObj(object, fn, options, seen) {
    options = Object.assign({
        deep: false,
        target: {}
    }, options);
    seen = seen || new WeakMap();
    if (seen.has(object)) {
        return seen.get(object);
    }
    seen.set(object, options.target);
    var target = options.target;
    delete options.target;
    var mapArray = function (array) {
        return array.map(function (x) { return (isObject(x) ? mapObj(x, fn, options, seen) : x); });
    };
    if (Array.isArray(object)) {
        return mapArray(object);
    }
    for (var _i = 0, _a = Object.keys(object); _i < _a.length; _i++) {
        var key = _a[_i];
        var value = object[key];
        var _b = fn(key, value, object), newKey = _b[0], newValue = _b[1];
        if (options.deep && isObject(newValue)) {
            newValue = Array.isArray(newValue)
                ? mapArray(newValue)
                : mapObj(newValue, fn, options, seen);
        }
        target[newKey] = newValue;
    }
    return target;
}

function set (obj, keys, val) {
    keys.split && (keys = keys.split('.'));
    var l = keys.length;
    var t = obj;
    var x;
    for (var i = 0; i < l; ++i) {
        x = t[keys[i]];
        if (typeof t !== 'object')
            continue;
        t = t[keys[i]] =
            i === l - 1
                ? val
                : x != null
                    ? x
                    : !!~keys[i + 1].indexOf('.') || !(+keys[i + 1] > -1)
                        ? {}
                        : [];
    }
}

var index$1 = { filter: filter, get: get, map: mapObj, set: set };

function padLeft(str, num, ch) {
    if (ch === void 0) { ch = ' '; }
    str = str.toString();
    if (typeof num === 'undefined') {
        return str;
    }
    if (ch !== ' ') {
        ch = ch.toString();
    }
    return ch.repeat(num - str.length) + str;
}
function padRight(str, num, ch) {
    if (ch === void 0) { ch = ' '; }
    return str + ch.repeat(num - str.length);
}

var digit = '0123456789';
var uppercase = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
var lowercase = 'abcdefghijklmnopqrstuvwxyz';
var alphabet = "" + uppercase + lowercase;
var url = digit + "_" + uppercase + "-" + lowercase;

function replaceAll(target, search, replacement) {
    return target.replace(new RegExp(search, 'g'), replacement);
}
var Str = {
    digit: digit,
    lowercase: lowercase,
    uppercase: uppercase,
    alphabet: alphabet,
    url: url,
    pad: padLeft,
    padLeft: padLeft,
    padRight: padRight,
    replaceAll: replaceAll
};

var MAX_INT32 = 2147483647;
var MINSTD = 16807;
var Random = (function () {
    function Random(option) {
        if (option === void 0) { option = {}; }
        this.prev = {};
        this.option = option;
        if (!Number.isInteger(this.option.seed)) {
            this.option.seed = this.randomSeed();
        }
        this.seed = this.option.seed % MAX_INT32;
        if (this.seed <= 0) {
            this.seed += MAX_INT32 - 1;
        }
    }
    Random.prototype.randomSeed = function () {
        return MAX_INT32 - Math.floor(Math.random() * 10000);
    };
    Random.prototype.int = function (min, max) {
        var num = Number.isInteger(min)
            ? this.integerInRange(min, max)
            : this.integer();
        this.prev.int =
            this.option.unique && num === this.prev.int ? this.int(min, max) : num;
        return this.prev.int;
    };
    Random.prototype.integer = function () {
        this.seed *= MINSTD;
        this.seed %= MAX_INT32;
        return this.seed;
    };
    Random.prototype.integerInRange = function (min, max) {
        return Math.round(this.floatInRange(min, max));
    };
    Random.prototype.float = function (min, max) {
        var num = typeof min !== 'number' ? this.randomFloat() : this.floatInRange(min, max);
        this.prev.float =
            this.option.unique && num === this.prev.float ? this.float(min, max) : num;
        return this.prev.float;
    };
    Random.prototype.randomFloat = function () {
        return (this.integer() - 1) / (MAX_INT32 - 1);
    };
    Random.prototype.floatInRange = function (min, max) {
        if (typeof max !== 'number') {
            max = MAX_INT32;
        }
        return min + (max - min) * this.randomFloat();
    };
    Random.prototype.boolean = function () {
        return this.integer() % 2 === 0;
    };
    Random.prototype.randomSize = function () {
        return this.int(1, 32);
    };
    Random.prototype.generate = function (alphabet, size) {
        if (!Number.isInteger(size)) {
            size = this.randomSize();
        }
        var id = '';
        var randomMax = alphabet.length - 1;
        while (true) {
            id += alphabet[this.int(0, randomMax)];
            if (id.length === size)
                break;
        }
        var key = '_' + alphabet;
        this.prev[key] =
            this.option.unique && id === this.prev[key]
                ? this.generate(alphabet, size)
                : id;
        return this.prev[key];
    };
    Random.prototype.string = function (str, size) {
        if (Number.isInteger(str)) {
            size = str;
        }
        if (typeof str !== 'string') {
            str = Str.url;
        }
        return this.generate(str, size);
    };
    Random.prototype.lower = function (size) {
        return this.generate(Str.lowercase, size);
    };
    Random.prototype.upper = function (size) {
        return this.generate(Str.uppercase, size);
    };
    Random.prototype.alphabet = function (size) {
        return this.generate(Str.alphabet, size);
    };
    Random.prototype.digit = function (size) {
        return this.generate(Str.digit, size);
    };
    Random.prototype.array = function (arr) {
        var item = arr[this.int(0, arr.length - 1)];
        this.prev.array =
            this.option.unique && item === this.prev.array ? this.array(arr) : item;
        return this.prev.array;
    };
    return Random;
}());

function Wrandom(data, cb) {
    var ran = Math.random();
    var acc = 0;
    var weights = data;
    var returnCb = function (i) { return i; };
    if (Array.isArray(data)) {
        if (typeof weights[0] !== 'number') {
            weights = data.map(function (v) { return (cb ? cb(v) : v.weight); });
            returnCb = function (i) { return data[i]; };
        }
    }
    else if (typeof data === 'object') {
        var entries_1 = Object.entries(data);
        weights = entries_1.map(function (v) { return v[1]; });
        returnCb = function (i) { return entries_1[i][0]; };
    }
    var ratio = 1 / weights.reduce(function (a, b) { return a + b; });
    for (var i = 0; i < weights.length; i++) {
        acc += weights[i] * ratio;
        if (ran < acc)
            return returnCb(i);
    }
}

exports.Arr = Arr;
exports.Hash = Hash;
exports.Is = index;
exports.Obj = index$1;
exports.Random = Random;
exports.Str = Str;
exports.Wrandom = Wrandom;
